SERVICE_NAME = 'polly'
BASE_URL_TEMPLATE = 'https://{service_name}.{region}.amazonaws.com'

CHARACTERS_HEADER = 'x-amzn-RequestCharacters'
EXCEPTION_HEADER = 'x-amzn-ErrorType'

TRUST_API_RESPONSES = False
CONVERT_TO_SNAKE_CASE = True
